 /*
	* Copyright 2011-2012 jullichten < jul@siilia.fr >
	*
	* This program is free software: you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation, either version 3 of the License, or
	* (at your option) any later version.
	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*
	* You should have received a copy of the GNU General Public License
	* along with this program.  If not, see <http://www.gnu.org/licenses/>.
	*/
const Main = imports.ui.main;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Panel = Main.panel;
const St = imports.gi.St;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Shell = imports.gi.Shell;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Meta = imports.gi.Meta;
const Convenience = Me.imports.convenience;
const _ = imports.gettext.domain("gnome-shell-extension-mouse-travel").gettext;

// aka wait between two samples
const UPDATE_INTERVAL = 200;
// save travel every ... ms
const SAVE_INTERVAL = 10000;

// globals
let mtr;
let extensionPath;

// settings
let settingHandles = [];

function MouseTravel() { this._init.apply (this, arguments); }
MouseTravel.prototype = {
	__proto__: PanelMenu.Button.prototype,

	unload: function()
	{
		try {
		if (this._update_handler) Mainloop.source_remove(this._update_handler);
		if (this._save_handler) Mainloop.source_remove(this._save_handler);

		this._on_save();

		if (this._schema !== null && settingHandles !== null) {
			for (let i = 0; i < settingHandles.length; i++) {
				if (settingHandles[i] > -1) this._schema.disconnect(settingHandles[i]);
			}
		}
		
		} catch(e) { Main.notify(e.message); }
	},

	_init: function () {
		try {
		PanelMenu.Button.prototype._init.call( this, St.Align.START );
		
		this._box = new St.BoxLayout({ style_class: 'panel-status-button-box' });

		// load ressources...
		this.icon = new St.Icon({
			icon_name: 'mouse-travel-symbolic',
			style_class: 'popup-menu-icon '
		});
		this.info = new St.Label({
			text: "--"
		});

		this._box.insert_child_at_index(this.icon, 0);
		this._box.insert_child_at_index(this.info, 1);

		// load custom icon
		this.icon.set_gicon(
			new Gio.FileIcon({
				file: Gio.file_new_for_path( extensionPath + '/mouse-travel-symbolic.svg' )
			})
		);

		// menu
		this.infoMenuItem = new PopupMenu.PopupBaseMenuItem("");
		this.menu.addMenuItem(this.infoMenuItem);
		this.optMenuItem = new PopupMenu.PopupMenuItem(_("Settings"));
		this.menu.addMenuItem(this.optMenuItem);
		this.optMenuItem.connect("activate", Lang.bind(this, this._launchPrefs));		
		this.infoLabel = new St.Label ({text: ""});
		this.infoLabel.set_style("text-align:left;font-style:italic;");
		this.infoMenuItem.addActor( this.infoLabel );
		
		// init stuff
		this.pixel_size = 0.28;
		let[mouse_x, mouse_y, mask] = global.get_pointer();
		this.ox = mouse_x;
		this.oy = mouse_y;
		this.l = 0;
		this.lmm = 0;
		this.d = 0;

		this._schema = Convenience.getSettings();
		this.l = this._schema.get_uint("length");
		this.pixel_size = this._schema.get_double("pixelsize");

		// monitor settings changes
		settingHandles[0] = this._schema.connect("changed::pixelsize",
			Lang.bind(this, function() {
				this.pixel_size = this._schema.get_double("pixelsize");
			}));

		this._update_handler = Mainloop.timeout_add(UPDATE_INTERVAL, Lang.bind(this, this._on_timeout));
		this._save_handler = Mainloop.timeout_add(SAVE_INTERVAL, Lang.bind(this, this._on_save));

		this.actor.add_actor(this._box);
		Panel.addToStatusArea('mtr', this);
		} catch(e) { Main.notify(e.message); }
	},
	
	_get_travel: function() {
		if (this.lmm < 1000) return Math.round(this.lmm) + "mm";
		if (this.lmm < 1000 * 1000) return (Math.round((this.lmm / 1000) * 10) / 10) + "m";

		return (Math.floor(this.lmm / 100000) * 10) / 100 + "km";
	},
	
	_get_full_travel: function() {
		
		let km = Math.floor(this.lmm/1000000);
		let m = Math.floor((this.lmm-(km*1000000))/1000);
		let cm = Math.floor((this.lmm-((km*1000000)+(m*1000)))/10);
		return km+"km "+m+"m "+cm+"cm";
	},

	_on_save: function() {
		this._schema.set_uint("length", this.l);
		return true;
	},
		
	_on_timeout: function() {
		// get mouse position
		let[mouse_x, mouse_y, mask] = global.get_pointer();
		// First pass.. no move
		if (this.ox == 0) {
			this.ox = mouse_x;
			return true;
		}
		// If no move since last call, quit.
		if (mouse_x == this.ox && mouse_y == this.oy)	return true;

		// we moved in he last UPDATE_INTERVAL ms
		this.d += UPDATE_INTERVAL;

		// compute distance
		tx = Math.abs((mouse_x - this.ox));
		ty = Math.abs((mouse_y - this.oy));
		cl = Math.round(Math.sqrt((tx * tx) + (ty * ty)));
		this.l += cl
		this.lmm = this.l * this.pixel_size;

		// update label
		this.info.set_text(this._get_travel());
		//this.infoLabel.set_tooltip_text(this._get_full_travel());

		// update full info
		this.infoLabel.set_text( this._get_full_travel() );

		// save for next timeout...
		this.ox = mouse_x;
		this.oy = mouse_y;
		return true;
	},
	_launchPrefs: function() {
		let appSys = Shell.AppSystem.get_default();
		let app = appSys.lookup_app('gnome-shell-extension-prefs.desktop');
		app.launch(global.display.get_current_time_roundtrip(), ['extension:///mouse-travel@siilia.fr'], -1, null);
		this.menu.close();
	}	
}

/********************* Extension Creator ********************/
function init(meta)
{
	extensionPath = meta.path;
	Convenience.initTranslations("gnome-shell-extension-mouse-travel");
}

function enable()
{
	mtr = new MouseTravel();
}

function disable()
{
	mtr.unload();
	mtr.destroy();
}
